@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008-2013 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2008-2014 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -42,14 +42,25 @@ def clock(prefix, operation, *args, **kwargs):
     notify('{0} ({1:0.2f}s)'.format(prefix, stop - go))
     return result
 
-def run(skoolfile, be_quiet, properties, parser_mode, writer_mode):
+def run(skoolfile, options, parser_mode, writer_mode):
     global quiet
-    quiet = be_quiet
+    quiet = options.quiet
 
     # Create the parser
     case, base, asm_mode, warn, fix_mode, create_labels = parser_mode
-    parser = clock('Parsed {0}'.format(skoolfile), SkoolParser, skoolfile, case=case, base=base, asm_mode=asm_mode, warnings=warn, fix_mode=fix_mode, create_labels=create_labels)
+    if skoolfile == '-':
+        fname = 'stdin'
+    else:
+        fname = skoolfile
+    parser = clock('Parsed {}'.format(fname), SkoolParser, skoolfile, case=case, base=base, asm_mode=asm_mode, warnings=warn, fix_mode=fix_mode, create_labels=create_labels)
 
+    properties = {}
+    if options.crlf:
+        properties[CRLF] = '1'
+    if options.inst_width is not None:
+        properties[INSTRUCTION_WIDTH] = options.inst_width
+    if options.tabs:
+        properties[TAB] = '1'
     writer_properties = {}
     writer_properties.update(parser.properties)
     writer_properties.update(properties)
@@ -64,7 +75,7 @@ def run(skoolfile, be_quiet, properties, parser_mode, writer_mode):
         instr_width = DEF_INSTR_WIDTH
 
     # Write the ASM file
-    cls_name = parser.asm_writer_class
+    cls_name = options.writer or parser.asm_writer_class
     if cls_name:
         asm_writer_class = get_class(cls_name)
         notify('Using ASM writer {0}'.format(cls_name))
@@ -87,7 +98,7 @@ def main(args):
                        help="Create default labels for unlabelled instructions")
     group.add_argument('-d', '--crlf', dest='crlf', action='store_true',
                        help="Use CR+LF to end lines")
-    group.add_argument('-D', '--decimal', dest='decimal', action='store_true',
+    group.add_argument('-D', '--decimal', dest='base', action='store_const', const=BASE_10,
                        help="Write the disassembly in decimal")
     group.add_argument('-f', '--fixes', dest='fix_mode', metavar='N', type=int, choices=range(4), default=0,
                        help="Apply fixes:\n"
@@ -95,56 +106,36 @@ def main(args):
                             "  N=1: @ofix only\n"
                             "  N=2: @ofix and @bfix\n"
                             "  N=3: @ofix, @bfix and @rfix (implies -r)")
-    group.add_argument('-H', '--hex', dest='hex', action='store_true',
+    group.add_argument('-H', '--hex', dest='base', action='store_const', const=BASE_16,
                        help="Write the disassembly in hexadecimal")
     group.add_argument('-i', '--inst-width', dest='inst_width', metavar='N', type=int,
                        help="Set instruction field width (default={})".format(DEF_INSTR_WIDTH))
-    group.add_argument('-l', '--lower', dest='lower', action='store_true',
+    group.add_argument('-l', '--lower', dest='case', action='store_const', const=CASE_LOWER,
                        help="Write the disassembly in lower case")
     group.add_argument('-p', '--package-dir', dest='package_dir', action='store_true',
                        help="Show path to skoolkit package directory and exit")
     group.add_argument('-q', '--quiet', dest='quiet', action='store_true',
                        help="Be quiet")
-    group.add_argument('-r', '--rsub', dest='rsub', action='store_true',
+    group.add_argument('-r', '--rsub', dest='asm_mode', action='store_const', const=3, default=1,
                        help="Use relocatability substitutions too (@rsub) (implies\n'-f 1')")
-    group.add_argument('-s', '--ssub', dest='ssub', action='store_true',
+    group.add_argument('-s', '--ssub', dest='asm_mode', action='store_const', const=2, default=1,
                        help="Use safe substitutions (@ssub)")
     group.add_argument('-t', '--tabs', dest='tabs', action='store_true',
                        help="Use tab to indent instructions (default indentation is\n2 spaces)")
-    group.add_argument('-u', '--upper', dest='upper', action='store_true',
+    group.add_argument('-u', '--upper', dest='case', action='store_const', const=CASE_UPPER,
                        help="Write the disassembly in upper case")
     group.add_argument('-V', '--version', action='version', version='SkoolKit {}'.format(VERSION),
                        help='Show SkoolKit version number and exit')
     group.add_argument('-w', '--no-warnings', dest='warn', action='store_false',
                        help="Suppress warnings")
+    group.add_argument('-W', '--writer', dest='writer', metavar='CLASS',
+                       help="Specify the ASM writer class to use")
 
     namespace, unknown_args = parser.parse_known_args(args)
     if namespace.package_dir:
         show_package_dir()
     if unknown_args or namespace.skoolfile is None:
         parser.exit(2, parser.format_help())
-    properties = {}
-    case = None
-    base = None
-    asm_mode = 1
-    if namespace.crlf:
-        properties[CRLF] = '1'
-    if namespace.inst_width is not None:
-        properties[INSTRUCTION_WIDTH] = namespace.inst_width
-    if namespace.tabs:
-        properties[TAB] = '1'
-    if namespace.lower:
-        case = CASE_LOWER
-    elif namespace.upper:
-        case = CASE_UPPER
-    if namespace.decimal:
-        base = BASE_10
-    elif namespace.hex:
-        base = BASE_16
-    if namespace.rsub:
-        asm_mode = 3
-    elif namespace.ssub:
-        asm_mode = 2
-    parser_mode = (case, base, asm_mode, namespace.warn, namespace.fix_mode, namespace.create_labels)
-    writer_mode = (case == CASE_LOWER, namespace.warn)
-    run(namespace.skoolfile, namespace.quiet, properties, parser_mode, writer_mode)
+    parser_mode = (namespace.case, namespace.base, namespace.asm_mode, namespace.warn, namespace.fix_mode, namespace.create_labels)
+    writer_mode = (namespace.case == CASE_LOWER, namespace.warn)
+    run(namespace.skoolfile, namespace, parser_mode, writer_mode)
