@@ -1,13 +1,18 @@
 # -*- coding: utf-8 -*-
+import sys
 import zlib
 import unittest
 from collections import deque
 from io import BytesIO
 
-from skoolkittest import SkoolKitTestCase, PY3
-from skoolkit.image import ImageWriter, PngWriter, DEFAULT_FORMAT, PNG_COMPRESSION_LEVEL, PNG_ENABLE_ANIMATION, PNG_ALPHA, GIF_ENABLE_ANIMATION, GIF_TRANSPARENCY, GIF_COMPRESSION
+from skoolkittest import SkoolKitTestCase
+from skoolkit.image import (ImageWriter, PngWriter, DEFAULT_FORMAT,
+                            PNG_COMPRESSION_LEVEL, PNG_ENABLE_ANIMATION,
+                            PNG_ALPHA, GIF_ENABLE_ANIMATION, GIF_TRANSPARENCY)
 from skoolkit.skoolhtml import Udg, Frame
 
+PY3 = sys.version_info >= (3,)
+
 TRANSPARENT = [0, 254, 0]
 BLACK = [0, 0, 0]
 BLUE = [0, 0, 197]
@@ -92,6 +97,12 @@ def create_attr_index():
 CRC_TABLE = create_crc_table()
 ATTR_INDEX = create_attr_index()
 
+class MockFrame:
+    def __init__(self, tiles, num_attrs, scale):
+        self.tiles = tiles
+        self.attr_map = dict([(i, 0) for i in range(num_attrs)])
+        self.scale = scale
+
 class ImageWriterOptionsTest(SkoolKitTestCase):
     def test_change_option_values(self):
         options = {
@@ -145,115 +156,133 @@ class ImageWriterTest:
         img_stream.close()
         return img_bytes
 
-    def _get_pixels_from_udg_array(self, udg_array, scale, mask, x, y, width, height):
+    def _get_pixels_from_udg_array(self, udg_array, scale, mask, x0=0, y0=0, width=None, height=None):
         full_width = 8 * len(udg_array[0]) * scale
         full_height = 8 * len(udg_array) * scale
-        width = min(width or full_width, full_width - x)
-        height = min(height or full_height, full_height - y)
+        width = min(width or full_width, full_width - x0)
+        height = min(height or full_height, full_height - y0)
+        cropped = width < full_width or height < full_height
+        x1 = x0 + width
+        y1 = y0 + height
+        inc = 8 * scale
+        min_col = x0 // inc
+        max_col = x1 // inc
+        min_row = y0 // inc
+        max_row = y1 // inc
+        x0_floor = inc * min_col
+        y0_floor = inc * min_row
 
         palette = []
         pixels = []
         pixels2 = []
-        has_trans = 0
-        all_trans = 1
-        y_count = 0
-        inc = 8 * scale
-
+        mask = int(mask)
+        has_masks = 0
+        all_masked = 1
         min_x = width
         min_y = height
         max_x = max_y = 0
+        frames_differ = False
 
-        for row in udg_array:
-            if y_count >= y + height:
-                break
-            if y_count + inc <= y:
-                y_count += inc
-                continue
-            for j in range(8):
-                if y_count + scale <= y:
-                    y_count += scale
-                    continue
-                if y_count >= y:
-                    num_lines = min(y + height - y_count, scale)
+        y = y0_floor
+        for row in udg_array[min_row:max_row + 1]:
+            min_j = max(0, (y0 - y) // scale)
+            max_j = min(8, 1 + (y1 - y - 1) // scale)
+            y += min_j * scale
+            for j in range(min_j, max_j):
+                if y < y0:
+                    rows = y - y0 + scale
                 else:
-                    num_lines = y_count - y + scale
+                    rows = min(y1 - y, scale)
                 pixel_row = []
                 pixel_row2 = []
-                x_count = 0
-                for udg in row:
-                    if x_count >= x + width:
-                        break
-                    if x_count + inc <= x:
-                        x_count += inc
-                        continue
+                x = x0_floor
+                for udg in row[min_col:max_col + 1]:
                     attr = udg.attr
                     paper, ink = ATTR_INDEX[attr & 127]
-                    flashing = attr & 128 and paper != ink
                     p_rgb = PALETTE[paper]
                     i_rgb = PALETTE[ink]
                     byte = udg.data[j]
+                    if udg.mask:
+                        has_masks = 1
+                    else:
+                        all_masked = 0
                     if mask and udg.mask:
                         mask_byte = udg.mask[j]
-                        byte &= mask_byte
                     else:
-                        all_trans = 0
                         mask_byte = 0
-                    for k in range(8):
-                        if x_count >= x + width:
-                            break
-                        if x_count + scale <= x:
-                            x_count += scale
-                            byte *= 2
-                            mask_byte *= 2
-                            continue
-                        if x_count >= x:
-                            num_bits = min(x + width - x_count, scale)
+                    min_k = max(0, (x0 - x) // scale)
+                    max_k = min(8, 1 + (x1 - x - 1) // scale)
+                    x += min_k * scale
+                    flashing = attr & 128 and paper != ink
+                    has_non_trans = False
+                    byte <<= min_k
+                    mask_byte <<= min_k
+                    min_x_floor = len(pixel_row)
+                    for k in range(min_k, max_k):
+                        if x < x0:
+                            cols = x - x0 + scale
                         else:
-                            num_bits = x_count - x + scale
+                            cols = min(x1 - x, scale)
+                        ink_p = (i_rgb,) * cols
+                        paper_p = (p_rgb,) * cols
+                        trans_p = (TRANSPARENT,) * cols
+                        if mask == 1 and udg.mask:
+                            if mask_byte & 128 == 0:
+                                pixel, f_pixel = paper_p, ink_p
+                            elif byte & 128:
+                                pixel, f_pixel = ink_p, paper_p
+                            else:
+                                pixel = f_pixel = trans_p
+                        elif mask == 2 and udg.mask:
+                            if byte & 128:
+                                pixel, f_pixel = ink_p, paper_p
+                            elif mask_byte & 128:
+                                pixel = f_pixel = trans_p
+                            else:
+                                pixel, f_pixel = paper_p, ink_p
+                        else:
+                            if byte & 128:
+                                pixel, f_pixel = ink_p, paper_p
+                            else:
+                                pixel, f_pixel = paper_p, ink_p
+                        pixel_rgb = pixel[0]
+                        if pixel_rgb != TRANSPARENT:
+                            has_non_trans = True
+                        if pixel_rgb not in palette:
+                            palette.append(pixel_rgb)
+                        pixel_row.extend(pixel)
                         if flashing:
-                            min_x = min((len(pixel_row), min_x))
-                            max_x = max((len(pixel_row) + num_bits, max_x))
-                            min_y = min((len(pixels), min_y))
-                            max_y = max((len(pixels) + num_lines, max_y))
-                        if byte & 128:
-                            pixel_row.extend((i_rgb,) * num_bits)
-                            if flashing:
-                                pixel_row2.extend((p_rgb,) * num_bits)
-                            else:
-                                pixel_row2.extend((i_rgb,) * num_bits)
-                        elif mask_byte & 128:
-                            pixel_row.extend((TRANSPARENT,) * num_bits)
-                            pixel_row2.extend((TRANSPARENT,) * num_bits)
-                            has_trans = 1
+                            f_pixel_rgb = f_pixel[0]
+                            if f_pixel_rgb not in palette:
+                                palette.append(f_pixel_rgb)
+                            pixel_row2.extend(f_pixel)
+                            frames_differ = frames_differ or f_pixel != pixel
                         else:
-                            pixel_row.extend((p_rgb,) * num_bits)
-                            if flashing:
-                                pixel_row2.extend((i_rgb,) * num_bits)
-                            else:
-                                pixel_row2.extend((p_rgb,) * num_bits)
+                            pixel_row2.extend(pixel)
                         byte *= 2
                         mask_byte *= 2
-                        x_count += scale
-                for pixel in pixel_row:
-                    if pixel not in palette:
-                        palette.append(pixel)
-                for pixel in pixel_row2:
-                    if pixel not in palette:
-                        palette.append(pixel)
-                for n in range(num_lines):
-                    pixels.append(pixel_row)
-                    pixels2.append(pixel_row2)
-                y_count += scale
+                        x += scale
+                    if flashing and (has_non_trans or cropped):
+                        # A 'flashing' cell that is entirely transparent is
+                        # excluded from a full-size image, but included in a
+                        # cropped image
+                        min_x = min(min_x_floor, min_x)
+                        max_x = max(len(pixel_row), max_x)
+                        min_y = min(len(pixels), min_y)
+                        max_y = max(len(pixels) + rows, max_y)
+                pixels += [pixel_row] * rows
+                pixels2 += [pixel_row2] * rows
+                y += scale
 
-        if pixels2 == pixels:
-            pixels2 = frame2_xy = None
-        else:
+        if frames_differ:
             frame2_xy = (min_x, min_y)
             pixels2 = [pixels2[i][min_x:max_x] for i in range(min_y, max_y)]
+        else:
+            pixels2 = frame2_xy = None
 
-        trans = has_trans + all_trans if has_trans else 0
 
-        return palette, trans, pixels, pixels2, frame2_xy
+        masks = has_masks + all_masked
+        return palette, masks, pixels, pixels2, frame2_xy
 
     def test_masked_alpha(self):
         # Masked image, alpha < 255
@@ -262,6 +291,18 @@ class ImageWriterTest:
         udg_array = [[udg]]
         self._test_image(udg_array, mask=True, iw_args=iw_args)
 
+    def test_masked_no_transparency(self):
+        # Masked image with no transparent bits
+        udg = Udg(56, (255,) * 8, (1,) * 8)
+        udg_array = [[udg]]
+        self._test_image(udg_array, mask=1)
+
+    def test_masked_cropped_no_transparency(self):
+        # Masked image, cropped, with no transparent bits
+        udg = Udg(56, (255,) * 8, (1,) * 8)
+        udg_array = [[udg]]
+        self._test_image(udg_array, mask=1, y=2)
+
     def test_masked_bd1_blank_flashing(self):
         # Masked image, single colour, flashing
         udg = Udg(184, (0,) * 8, (255,) * 8)
@@ -275,17 +316,30 @@ class ImageWriterTest:
         self._test_image(udg_array, mask=True)
         self._test_image(udg_array, scale=2, mask=True)
 
+    def test_masked_bd1_flashing(self):
+        # Masked image, flashing, no transparent bits
+        udg = Udg(184, (1,) * 8, (1,) * 8)
+        udg_array = [[udg] * 4]
+        self._test_image(udg_array, scale=1, mask=1)
+
     def test_masked_bd1_ink(self):
-        # Masked image, two colours (trans + ink), >= 4 UDGs
+        # Masked image, two colours (trans + ink)
         udg = Udg(49, (136,) * 8, (255,) * 8)
-        udg_array = [[udg, udg]] * 2
+        udg_array = [[udg]]
         self._test_image(udg_array, mask=True)
 
     def test_masked_bd1_paper(self):
-        # Masked image, two colours (trans + paper), >= 4 UDGs
+        # Masked image, two colours (trans + paper)
         udg = Udg(8, (0,) * 8, (88,) * 8)
-        udg_array = [[udg, udg]] * 2
-        self._test_image(udg_array,  mask=True)
+        udg_array = [[udg]]
+        self._test_image(udg_array, mask=True)
+
+    def test_mask2_bd1_one_udg_maskless(self):
+        # AND-OR mask, one UDG with no mask
+        udg1 = Udg(7, (15,) * 8, (31,) * 8)
+        udg2 = Udg(7, (240,) * 8)
+        udg_array = [[udg1, udg2]]
+        self._test_image(udg_array, mask=2)
 
     def test_masked_bd2_cropped(self):
         # Masked image, bit depth 2, cropped
@@ -301,7 +355,20 @@ class ImageWriterTest:
         # Masked image, bit depth 2
         udg = Udg(88, (34,) * 8, (119,) * 8)
         udg_array = [[udg]]
-        self._test_image(udg_array, mask=True)
+        self._test_image(udg_array, scale=1, mask=True)
+
+    def test_mask2_bd2(self):
+        # AND-OR mask, bit depth 2
+        udg = Udg(56, (170,) * 8, (170,) + (255,) * 7)
+        udg_array = [[udg]]
+        self._test_image(udg_array, mask=2)
+        self._test_image(udg_array, scale=2, mask=2)
+
+    def test_mask2_bd2_cropped(self):
+        # AND-OR mask, bit depth 2, cropped
+        udg = Udg(56, (170,) * 8, (170,) + (255,) * 7)
+        udg_array = [[udg] * 3]
+        self._test_image(udg_array, mask=2, x=7, y=3, width=9, height=2)
 
     def test_masked_bd4_all_bp(self):
         # Masked image, bit depth 4, all bit patterns
@@ -340,7 +407,8 @@ class ImageWriterTest:
         udg1 = Udg(24, (34,) * 8, (119,) * 8)
         udg2 = Udg(57, (170,) * 8)
         udg_array = [[udg1, udg2]]
-        self._test_image(udg_array, scale=2, mask=True, x=1, y=3, width=27, height=12)
+        self._test_image(udg_array, scale=2, mask=True, x=1, y=3, width=25, height=11)
+        self._test_image(udg_array, scale=2, mask=True, x=1, y=3, width=26, height=12)
 
     def test_masked_cropped_flashing(self):
         # Masked image, cropped, flashing
@@ -349,12 +417,26 @@ class ImageWriterTest:
         udg_array = [[udg1, udg2]] * 2
         self._test_image(udg_array, scale=2, mask=True, x=1, y=1, width=29, height=27)
 
+    def test_masked_cropped_flashing_one_udg_all_transparent(self):
+        # Masked image, cropped, flashing, one UDG all transparent
+        udg1 = Udg(135, (0,) * 8, (255,) * 8)
+        udg2 = Udg(184, (240,) * 8)
+        udg_array = [[udg1, udg2]]
+        self._test_image(udg_array, scale=2, mask=True, x=1, y=1, width=27, height=13)
+
     def test_masked_flashing(self):
         # Masked image, flashing
         udg = Udg(184, (240,) * 8, (243,) * 8)
         udg_array = [[udg]]
         self._test_image(udg_array, mask=True)
 
+    def test_masked_flashing_one_udg_all_transparent(self):
+        # Masked image, flashing, one UDG all transparent
+        udg1 = Udg(135, (0,) * 8, (255,) * 8)
+        udg2 = Udg(135, (240,) * 8)
+        udg_array = [[udg1, udg2]]
+        self._test_image(udg_array, scale=2, mask=True)
+
     def test_masked_flashing_no_animation(self):
         # Masked image, flashing, no animation
         iw_args = {'options': {self.animation_flag: 0}}
@@ -408,15 +490,15 @@ class ImageWriterTest:
         self._test_image(udg_array, x=1, y=2, width=9, height=11)
 
     def test_unmasked_bd1_flashing(self):
-        # Unmasked image, bit depth 1, >3 UDGs
+        # Unmasked image, bit depth 1, 2 UDGs
         udg = Udg(177, (164,) * 8)
-        udg_array = [[udg] * 4]
+        udg_array = [[udg] * 2]
         self._test_image(udg_array)
 
     def test_unmasked_bd1_scaled(self):
-        # Unmasked image, bit depth 1, scales 1-2, >3 UDGs
+        # Unmasked image, bit depth 1, scales 1-2, 2 UDGs
         udg = Udg(49, (164,) * 8)
-        udg_array = [[udg] * 4]
+        udg_array = [[udg] * 2]
         self._test_image(udg_array)
         self._test_image(udg_array, scale=2)
         return udg_array
@@ -492,7 +574,7 @@ class ImageWriterTest:
         udg6 = Udg(74, (15,) * 8)  # bright blue, bright red
         udg7 = Udg(92, (15,) * 8)  # bright magenta, bright green
         udg8 = Udg(110, (15,) * 8) # bright cyan, bright yellow
-        udg_array = [[udg1, udg2, udg3, udg4, udg5, udg6, udg7, udg8, udg1]] * 2
+        udg_array = [[udg1, udg2, udg3, udg4, udg5, udg6, udg7, udg8]]
         self._test_image(udg_array)
 
     def test_unmasked_bd4_cropped(self):
@@ -535,6 +617,7 @@ class ImageWriterTest:
         udg_array = [[udg1, udg2, udg3]]
         self._test_image(udg_array)
         self._test_image(udg_array, scale=2)
+        return udg_array
 
     def test_unmasked_flashing(self):
         # Unmasked image, flashing
@@ -542,6 +625,12 @@ class ImageWriterTest:
         udg_array = [[udg]]
         self._test_image(udg_array)
 
+    def test_unmasked_cropped_flashing(self):
+        # Unmasked image, cropped, flashing
+        udg = Udg(184, (240,) * 8)
+        udg_array = [[udg] * 3]
+        self._test_image(udg_array, x=2, y=0, width=17, height=8)
+
     def test_unmasked_flashing_no_animation(self):
         # Unmasked image, flashing, no animation
         iw_args = {'options': {self.animation_flag: 0}}
@@ -600,7 +689,7 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
 
     def _check_signature(self, img_bytes):
         i = len(PNG_SIGNATURE)
-        self.assertEqual(img_bytes[:i], PNG_SIGNATURE)
+        self.assertEqual(PNG_SIGNATURE, img_bytes[:i])
         return i
 
     def _check_ihdr(self, img_bytes, index, exp_width, exp_height, exp_bit_depth):
@@ -609,7 +698,7 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         self.assertEqual(chunk_length, 13)
         ihdr_start = i
         i, chunk_type = self._get_chunk_type(img_bytes, i)
-        self.assertEqual(chunk_type, IHDR)
+        self.assertEqual(IHDR, chunk_type)
         i, image_width = self._get_dword(img_bytes, i)
         self.assertEqual(image_width, exp_width)
         i, image_height = self._get_dword(img_bytes, i)
@@ -635,10 +724,10 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         self.assertEqual(chunk_length, len(exp_palette) * 3)
         plte_start = i
         i, chunk_type = self._get_chunk_type(img_bytes, i)
-        self.assertEqual(chunk_type, PLTE)
+        self.assertEqual(PLTE, chunk_type)
         palette = [img_bytes[j:j + 3] for j in range(i, i + chunk_length, 3)]
         i += chunk_length
-        self.assertEqual(sorted(palette), sorted(exp_palette))
+        self.assertEqual(sorted(exp_palette), sorted(palette))
         plte_end = i
         i, plte_crc = self._get_dword(img_bytes, i)
         self.assertEqual(plte_crc, self._get_crc(img_bytes[plte_start:plte_end]))
@@ -665,7 +754,7 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         self.assertEqual(chunk_length, 26)
         fctl_start = i
         i, chunk_type = self._get_chunk_type(img_bytes, i)
-        self.assertEqual(chunk_type, FCTL)
+        self.assertEqual(FCTL, chunk_type)
         i, frame_num = self._get_dword(img_bytes, i)
         self.assertEqual(frame_num, exp_frame_num)
         i, width = self._get_dword(img_bytes, i)
@@ -724,7 +813,7 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         i, chunk_length = self._get_dword(img_bytes, i)
         actl_start = i
         i, chunk_type = self._get_chunk_type(img_bytes, i)
-        self.assertEqual(chunk_type, ACTL)
+        self.assertEqual(ACTL, chunk_type)
         actl_end = i + chunk_length
         i, num_frames = self._get_dword(img_bytes, i)
         self.assertEqual(num_frames, exp_num_frames)
@@ -739,7 +828,7 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         i, chunk_length = self._get_dword(img_bytes, i)
         fdat_start = i
         i, chunk_type = self._get_chunk_type(img_bytes, i)
-        self.assertEqual(chunk_type, FDAT)
+        self.assertEqual(FDAT, chunk_type)
         fdat_end = i + chunk_length
         i, seq_num = self._get_dword(img_bytes, i)
         self.assertEqual(seq_num, exp_seq_num)
@@ -762,7 +851,7 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         i, chunk_length = self._get_dword(img_bytes, i)
         idat_start = i
         i, chunk_type = self._get_chunk_type(img_bytes, i)
-        self.assertEqual(chunk_type, IDAT)
+        self.assertEqual(IDAT, chunk_type)
         idat_end = i + chunk_length
         if PY3:
             image_data_z = bytes(img_bytes[i:idat_end])
@@ -773,27 +862,28 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         pixels = self._get_pixels_from_image_data(bit_depth, palette, image_data, width)
         self.assertEqual(len(pixels[0]), len(exp_pixels[0])) # width
         self.assertEqual(len(pixels), len(exp_pixels)) # height
-        self.assertEqual(pixels, exp_pixels)
+        self.assertEqual(exp_pixels, pixels)
         i, idat_crc = self._get_dword(img_bytes, idat_end)
         self.assertEqual(idat_crc, self._get_crc(img_bytes[idat_start:idat_end]))
         return i
 
-    def _test_method(self, method_name, udg_array, scale=1, mask=False, flash=0, x=0, y=0, width=None, height=None):
+    def _test_method(self, method_name, udg_array, scale=1, mask=0):
         image_writer = ImageWriter()
         png_writer = image_writer.writers['png']
         method = getattr(png_writer, '_build_image_data_{0}'.format(method_name))
 
         use_flash = image_writer.options[PNG_ENABLE_ANIMATION]
-        frame = Frame(udg_array, scale, mask, x, y, width, height)
+        frame = Frame(udg_array, scale, mask)
         width, height = frame.width, frame.height
         colours, attrs, flash_rect = image_writer._get_colours(frame, use_flash)
-        trans = frame.trans
-        palette, attr_map = image_writer._get_palette(colours, attrs, trans)
+        masks = frame.has_masks + frame.all_masked
+        palette, attr_map = image_writer._get_palette(colours, attrs, frame.has_trans)
+        frame.attr_map = attr_map
         palette = [palette[i:i + 3] for i in range(0, len(palette), 3)]
 
-        exp_palette, exp_trans, exp_pixels, exp_pixels2, frame2_xy = self._get_pixels_from_udg_array(udg_array, scale, mask, x, y, width, height)
+        exp_palette, exp_masks, exp_pixels, exp_pixels2, frame2_xy = self._get_pixels_from_udg_array(udg_array, scale, mask)
         self.assertEqual(sorted(palette), sorted(exp_palette))
-        self.assertEqual(trans, exp_trans)
+        self.assertEqual(masks, exp_masks)
         self.assertEqual(flash_rect, frame2_xy)
 
         palette_size = len(palette)
@@ -803,7 +893,7 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
             bit_depth = 2
         else:
             bit_depth = 1
-        image_data_z = method(udg_array, scale, attr_map, trans, flash, x, y, width, height, bit_depth)
+        image_data_z = method(frame, bit_depth=bit_depth, mask=png_writer.masks[mask])
         if PY3:
             image_data = list(zlib.decompress(image_data_z))
         else:
@@ -928,7 +1018,7 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         self.assertEqual(img_bytes[i:], IEND_CHUNK)
 
     def test_unmasked_bd1_scaled(self):
-        # Unmasked image, bit depth 1, scales 1-13, >3 UDGs
+        # Unmasked image, bit depth 1, scales 1-13, 2 UDGs
         udg_array = ImageWriterTest.test_unmasked_bd1_scaled(self)
         self._test_image(udg_array, scale=3)
         self._test_image(udg_array, scale=4)
@@ -941,6 +1031,9 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         self._test_image(udg_array, scale=11)
         self._test_image(udg_array, scale=12)
         self._test_image(udg_array, scale=13)
+        self._test_image(udg_array, scale=14)
+        self._test_image(udg_array, scale=15)
+        self._test_image(udg_array, scale=16)
 
     def test_unmasked_bd2_flashing(self):
         # Unmasked image, bit depth 2, flashing, scales 1-5
@@ -956,66 +1049,50 @@ class PngWriterTest(SkoolKitTestCase, ImageWriterTest):
         self._test_image(udg_array, scale=4)
         self._test_image(udg_array, scale=5)
 
-    def test_bd4_nt_nf(self):
+    def test_unmasked_bd4_scaled(self):
+        # Unmasked image, bit depth 4, scales 1-3
+        udg_array = ImageWriterTest.test_unmasked_bd4_scaled(self)
+        self._test_image(udg_array, scale=3)
+
+    def test_bd4_nt1(self):
         udg_array = [[Udg(attr, (15,) * 8) for attr in (1, 19, 37, 55)]]
-        method_name = 'bd4_nt_nf'
+        method_name = 'bd4_nt1'
         self._test_method(method_name, udg_array, scale=1)
         self._test_method(method_name, udg_array, scale=2)
         self._test_method(method_name, udg_array, scale=3)
         self._test_method(method_name, udg_array, scale=4)
 
-    def test_bd2_nt_nf(self):
+    def test_bd2_nt(self):
         udg_array = [[Udg(attr, (85,) * 8) for attr in (1, 19)]]
-        method_name = 'bd2_nt_nf'
+        method_name = 'bd2_nt'
         self._test_method(method_name, udg_array, scale=1)
         self._test_method(method_name, udg_array, scale=2)
         self._test_method(method_name, udg_array, scale=3)
         self._test_method(method_name, udg_array, scale=4)
         self._test_method(method_name, udg_array, scale=5)
 
-    def test_bd2_at_nf(self):
+    def test_bd2_at(self):
         udg_array = [[Udg(56, (15,) * 8, (207,) * 8)]]
-        method_name = 'bd2_at_nf'
+        method_name = 'bd2_at'
         self._test_method(method_name, udg_array, scale=2, mask=True)
         self._test_method(method_name, udg_array, scale=4, mask=True)
 
-    def test_bd1_nt_nf_1udg(self):
+    def test_bd1_nt_1udg(self):
         udg_array = [[Udg(34, (170,) * 8)]]
-        method_name = 'bd1_nt_nf_1udg'
+        method_name = 'bd1_nt_1udg'
         self._test_method(method_name, udg_array, scale=1)
         self._test_method(method_name, udg_array, scale=2)
         self._test_method(method_name, udg_array, scale=4)
         self._test_method(method_name, udg_array, scale=8)
 
-    def test_bd1_method(self):
-        iw = PngWriter()
-        self.assertEquals(iw._bd1_method(5, 0, 0), iw._build_image_data_bd1)
-        self.assertEquals(iw._bd1_method(6, 0, 0), iw._build_image_data_bd1)
-        self.assertEquals(iw._bd1_method(4, 2, 0), iw._build_image_data_bd1)
-        self.assertEquals(iw._bd1_method(3, 2, 0), iw._build_image_data_bd1)
-        self.assertEquals(iw._bd1_method(2, 3, 0), iw._build_image_data_bd_any)
+    def test_bd4_nt_method(self):
+        png_writer = ImageWriter().writers['png']
+        frame = MockFrame(1000, 3, 3)
+        self.assertEqual(png_writer._build_image_data_bd4_nt1, png_writer._bd4_nt_method(frame))
 
-    def test_bd2_at_nf_method(self):
-        iw = PngWriter()
-        self.assertEquals(iw._bd2_at_nf_method(0, 0, 2), iw._build_image_data_bd2_at_nf)
-        self.assertEquals(iw._bd2_at_nf_method(0, 0, 4), iw._build_image_data_bd2_at_nf)
-        self.assertEquals(iw._bd2_at_nf_method(0, 0, 8), iw._build_image_data_bd2_at_nf)
-        self.assertEquals(iw._bd2_at_nf_method(0, 0, 3), iw._build_image_data_bd2)
-        self.assertEquals(iw._bd2_at_nf_method(0, 0, 5), iw._build_image_data_bd2)
-        self.assertEquals(iw._bd2_at_nf_method(0, 0, 6), iw._build_image_data_bd2)
-        self.assertEquals(iw._bd2_at_nf_method(0, 0, 3), iw._build_image_data_bd2)
-
-    def test_bd4_nt_nf_method(self):
-        iw = PngWriter()
-        self.assertEquals(iw._bd4_nt_nf_method(35, 5, 1), iw._build_image_data_bd4_nt_nf)
-        self.assertEquals(iw._bd4_nt_nf_method(12, 6, 1), iw._build_image_data_bd4)
-        self.assertEquals(iw._bd4_nt_nf_method(50, 5, 2), iw._build_image_data_bd4_nt_nf)
-        self.assertEquals(iw._bd4_nt_nf_method(16, 6, 2), iw._build_image_data_bd4)
-        self.assertEquals(iw._bd4_nt_nf_method(195, 5, 3), iw._build_image_data_bd4_nt_nf)
-        self.assertEquals(iw._bd4_nt_nf_method(233, 6, 3), iw._build_image_data_bd4)
-
-class GifWriterTest(ImageWriterTest):
+class GifWriterTest(SkoolKitTestCase, ImageWriterTest):
     def setUp(self):
+        SkoolKitTestCase.setUp(self)
         self.animation_flag = GIF_ENABLE_ANIMATION
         self.alpha_option = {GIF_TRANSPARENCY: 1}
 
@@ -1052,7 +1129,7 @@ class GifWriterTest(ImageWriterTest):
         i, aspect_ratio = self._get_num(img_bytes, i)
         self.assertEqual(aspect_ratio, 0)
         gct = [img_bytes[j:j + 3] for j in range(i, i + 3 * exp_palette_size, 3)]
-        self.assertEqual(sorted(gct), sorted(exp_palette))
+        self.assertEqual(sorted(exp_palette), sorted(gct))
         i += 3 * len(gct)
         full_gct_len = 1 << (1 + exp_gct_size)
         for n in range(full_gct_len - len(gct)):
@@ -1081,6 +1158,7 @@ class GifWriterTest(ImageWriterTest):
         return i
 
     def _get_pixels_from_image_data(self, lzw_data, min_code_size, palette, width):
+        num_clear_codes = 0
         clear_code = 1 << min_code_size
         stop_code = clear_code + 1
         init_d = {}
@@ -1115,6 +1193,7 @@ class GifWriterTest(ImageWriterTest):
                 d = init_d.copy()
                 code_size = min_code_size + 1
                 prefix = None
+                num_clear_codes += 1
                 continue
             elif code == stop_code:
                 # Found the STOP code
@@ -1141,9 +1220,9 @@ class GifWriterTest(ImageWriterTest):
             if len(pixels[-1]) == width:
                 pixels.append([])
             pixels[-1].append(palette[pixel])
-        return pixels
+        return pixels, num_clear_codes
 
-    def _check_image_data(self, img_bytes, index, width, height, palette, exp_min_code_size, exp_pixels):
+    def _check_image_data(self, img_bytes, index, width, height, palette, exp_min_code_size, exp_pixels, exp_clear_codes=None):
         i, min_code_size = self._get_num(img_bytes, index)
         self.assertEqual(min_code_size, exp_min_code_size)
 
@@ -1155,18 +1234,19 @@ class GifWriterTest(ImageWriterTest):
             lzw_data.extend(img_bytes[i:i + length])
             i += length
 
-        pixels = self._get_pixels_from_image_data(lzw_data, min_code_size, palette, width)
+        pixels, num_clear_codes = self._get_pixels_from_image_data(lzw_data, min_code_size, palette, width)
         self.assertEqual(len(pixels[0]), len(exp_pixels[0])) # width
         self.assertEqual(len(pixels), len(exp_pixels)) # height
-        self.assertEqual(pixels, exp_pixels)
+        self.assertEqual(exp_pixels, pixels)
+        if exp_clear_codes is not None:
+            self.assertEqual(num_clear_codes, exp_clear_codes)
 
         return i
 
-    def _test_image(self, udg_array, exp_pixels=None, scale=1, mask=False, x=0, y=0, width=None, height=None, iw_args=None):
+    def _test_image(self, udg_array, exp_pixels=None, scale=1, mask=False, x=0, y=0, width=None, height=None, iw_args=None, exp_clear_codes=None):
         if iw_args is None:
             iw_args = {}
         options = iw_args.setdefault('options', {})
-        options.update(self.iw_options)
         image_writer = ImageWriter(**iw_args)
         img_bytes = self._get_image_data(image_writer, udg_array, 'gif', scale, mask, x, y, width, height)
 
@@ -1186,16 +1266,13 @@ class GifWriterTest(ImageWriterTest):
 
         t_flag = 1 if image_writer.options[GIF_TRANSPARENCY] and has_trans else 0
 
-        if image_writer.options[GIF_COMPRESSION]:
-            palette_size = len(exp_palette)
-            if palette_size > 8:
-                exp_min_code_size = 4
-            elif palette_size > 4:
-                exp_min_code_size = 3
-            else:
-                exp_min_code_size = 2
+        palette_size = len(exp_palette)
+        if palette_size > 8:
+            exp_min_code_size = 4
+        elif palette_size > 4:
+            exp_min_code_size = 3
         else:
-            exp_min_code_size = 7
+            exp_min_code_size = 2
 
         # GIF header
         i = self._check_header(img_bytes)
@@ -1221,7 +1298,7 @@ class GifWriterTest(ImageWriterTest):
         i = self._check_image_descriptor(img_bytes, i, exp_width, exp_height)
 
         # Frame 1 image data
-        i = self._check_image_data(img_bytes, i, exp_width, exp_height, palette, exp_min_code_size, exp_pixels)
+        i = self._check_image_data(img_bytes, i, exp_width, exp_height, palette, exp_min_code_size, exp_pixels, exp_clear_codes)
 
         # Frame 2
         if exp_pixels2:
@@ -1239,7 +1316,6 @@ class GifWriterTest(ImageWriterTest):
         if iw_args is None:
             iw_args = {}
         options = iw_args.setdefault('options', {})
-        options.update(self.iw_options)
         image_writer = ImageWriter(**iw_args)
         img_bytes = self._get_animated_image_data(image_writer, frames, 'gif')
 
@@ -1257,16 +1333,13 @@ class GifWriterTest(ImageWriterTest):
 
         t_flag = 1 if image_writer.options[GIF_TRANSPARENCY] and has_trans else 0
 
-        if image_writer.options[GIF_COMPRESSION]:
-            palette_size = len(exp_palette)
-            if palette_size > 8:
-                exp_min_code_size = 4
-            elif palette_size > 4:
-                exp_min_code_size = 3
-            else:
-                exp_min_code_size = 2
+        palette_size = len(exp_palette)
+        if palette_size > 8:
+            exp_min_code_size = 4
+        elif palette_size > 4:
+            exp_min_code_size = 3
         else:
-            exp_min_code_size = 7
+            exp_min_code_size = 2
 
         # GIF header
         i = self._check_header(img_bytes)
@@ -1292,40 +1365,15 @@ class GifWriterTest(ImageWriterTest):
         # GIF trailer
         self.assertEqual(img_bytes[i], GIF_TRAILER)
 
-class UncompressedGifWriterTest(SkoolKitTestCase, GifWriterTest):
-    def setUp(self):
-        SkoolKitTestCase.setUp(self)
-        GifWriterTest.setUp(self)
-        self.iw_options = {GIF_COMPRESSION: 0}
-
-class CompressedGifWriterTest(SkoolKitTestCase, GifWriterTest):
-    def setUp(self):
-        SkoolKitTestCase.setUp(self)
-        GifWriterTest.setUp(self)
-        self.iw_options = {GIF_COMPRESSION: 1}
-
-    def _bin_digits(self, n):
-        digits = str(n & 1)
-        n >>= 1
-        while n:
-            digits = str(n & 1) + digits
-            n >>= 1
-        return digits
-
     def test_lzw_clear_code(self):
-        pixels = ''
-        for n in range(3510):
-            pixels += self._bin_digits(n)
-        pixels += '0' * (64 - (len(pixels) & 63))
-        num_udgs = len(pixels) // 64
+        pixels = ''.join(['{:b}'.format(n) for n in range(1639)]) + '0' * 17
         udgs = []
-        for i in range(num_udgs):
-            udg_data = []
-            for j in range(8):
-                index = 8 * (i + num_udgs * j)
-                udg_data.append(int(pixels[index:index + 8], 2))
-            udgs.append(Udg(56, udg_data))
-        self._test_image([udgs])
+        index = 0
+        while index < len(pixels):
+            udg_data = [int(pixels[j:j + 8], 2) for j in range(index, index + 64, 8)]
+            udgs.append(Udg((index // 64) & 127, udg_data))
+            index += 64
+        self._test_image([udgs], exp_clear_codes=2)
 
 if __name__ == '__main__':
     unittest.main()
