@@ -113,9 +113,7 @@ class Tap2SnaTest(SkoolKitTestCase):
     def test_option_V(self):
         for option in ('-V', '--version'):
             output, error = self.run_tap2sna(option, err_lines=True, catch_exit=0)
-            self.assertEqual(len(output), 0)
-            self.assertEqual(len(error), 1)
-            self.assertEqual(error[0], 'SkoolKit {}'.format(VERSION))
+            self.assertEqual(['SkoolKit {}'.format(VERSION)], output + error)
 
     def test_nonexistent_tap_file(self):
         tap_fname = 'test.tap'
